@@ -1,4 +1,8 @@
-import json
+# -*- coding: utf-8 -*-
+from __future__ import (absolute_import, division, print_function,
+    unicode_literals)  # Avoid breaking Python 3
+
+import uuid
 from threading import local
 
 import sqlalchemy as sqla
@@ -8,9 +12,13 @@ from sqlalchemy.ext.mutable import Mutable
 from sqlalchemy.types import TypeDecorator, TEXT
 from zope.sqlalchemy import ZopeTransactionExtension
 
+from ptah.util import json
+
 _base = declarative.declarative_base()
 _zte = ZopeTransactionExtension()
 _session = orm.scoped_session(orm.sessionmaker(extension=[_zte]))
+_session_maker = orm.sessionmaker()
+_sa_session = local()
 
 
 def get_base():
@@ -26,6 +34,46 @@ def reset_session():
     _session = orm.scoped_session(orm.sessionmaker(extension=[_zte]))
 
 
+class transaction(object):
+
+    def __init__(self, sa):
+        self.sa = sa
+
+    def __enter__(self):
+        global _sa_session
+
+        t = getattr(_sa_session, 'transaction', None)
+        if t is not None:
+            raise RuntimeError("Nested transactions are not allowed")
+
+        _sa_session.sa = self.sa
+        _sa_session.transaction = self
+
+        return self.sa
+
+    def __exit__(self, type, value, traceback):
+        global _sa_session
+        _sa_session.sa = None
+        _sa_session.transaction = None
+
+        if type is None:
+            try:
+                self.sa.commit()
+            except:
+                self.sa.rollback()
+                raise
+        else:
+            self.sa.rollback()
+
+
+def sa_session():
+    return transaction(_session_maker())
+
+
+def get_session_maker():
+    return _session_maker
+
+
 def get_session():
     """Return the central SQLAlchemy contextual session.
 
@@ -43,7 +91,7 @@ def get_session():
 
         ptah.get_session().configure(ext=[ptah._zte, ...])
     """
-    return _session
+    return getattr(_sa_session, 'sa', _session) or _session
 
 
 class QueryFreezer(object):
@@ -57,31 +105,40 @@ class QueryFreezer(object):
             lambda: Session.query(Content)
                 .filter(Content.__uri__ == sqla.sql.bindparam('parent')))
     """
-    _testing = False
 
     def __init__(self, builder):
+        self.id = uuid.uuid4().int
         self.builder = builder
-        self.data = local()
 
     def reset(self):
-        self.data = local()
+        pass
 
     def iter(self, **params):
-        data = self.data
-        if not hasattr(data, 'query') or self._testing:
-            data.query = self.builder()
-            data.mapper = data.query._mapper_zero_or_none()
-            data.querycontext = data.query._compile_context()
-            data.querycontext.statement.use_labels = True
-            data.stmt = data.querycontext.statement
+        sa = get_session()
+        try:
+            data = sa.__ptah_cache__
+        except AttributeError:
+            sa.__ptah_cache__ = data = {}
 
-        conn = data.query._connection_from_session(
-            mapper=data.mapper,
-            clause=data.stmt,
+        q = data.get(self.id, None)
+
+        if q is None:
+            query = self.builder()
+            mapper = query._mapper_zero_or_none()
+            querycontext = query._compile_context()
+            querycontext.statement.use_labels = True
+            stmt = querycontext.statement
+            data[self.id] = (query, mapper, querycontext, stmt)
+        else:
+            query, mapper, querycontext, stmt = q
+
+        conn = query._connection_from_session(
+            mapper=mapper,
+            clause=stmt,
             close_with_result=True)
 
-        result = conn.execute(data.stmt, **params)
-        return data.query.instances(result, data.querycontext)
+        result = conn.execute(stmt, **params)
+        return query.instances(result, querycontext)
 
     def one(self, **params):
         ret = list(self.iter(**params))
@@ -106,19 +163,29 @@ class QueryFreezer(object):
         return list(self.iter(**params))
 
 
+def set_jsontype_serializer(serializer):
+    JsonType.serializer = serializer
+
+
 class JsonType(TypeDecorator):
     """Represents an immutable structure as a json-encoded string."""
 
     impl = TEXT
+    serializer = json
+
+    def __init__(self, serializer=None, *args, **kw):
+        if serializer is not None:
+            self.serializer = serializer
+        super(JsonType, self).__init__(*args, **kw)
 
     def process_bind_param(self, value, dialect):
         if value is not None:
-            value = json.dumps(value)
+            value = self.serializer.dumps(value)
         return value
 
     def process_result_value(self, value, dialect):
         if value is not None:
-            value = json.loads(value)
+            value = self.serializer.loads(value)
         return value
 
 
@@ -166,20 +233,19 @@ class MutationDict(Mutable, dict):
         self.changed()
 
 
-def JsonDictType():
+def JsonDictType(serializer=None):
     """
     function which returns a SQLA Column Type suitable to store a Json dict.
 
     :returns: ptah.sqla.MutationDict
     """
-    return MutationDict.as_mutable(JsonType)
+    return MutationDict.as_mutable(JsonType(serializer=serializer))
 
 
-def JsonListType():
+def JsonListType(serializer=None):
     """
     function which returns a SQLA Column Type suitable to store a Json array.
 
     :returns: ptah.sqla.MutationList
     """
-
-    return MutationList.as_mutable(JsonType)
+    return MutationList.as_mutable(JsonType(serializer=serializer))
